@@ -6,22 +6,23 @@
  *    Miguel de Icaza  (miguel@gnu.org)
  *    Nat Friedman     (nat@nat.org)
  *    Peter Wainwright (prw@wainpr.demo.co.uk)
+ *    Michael Meeks    (michael@ximian.com)
  *
- * Copyright 1999,2001 Ximian, Inc.
+ * Copyright 1999,2003 Ximian, Inc.
  */
 #include <config.h>
-
 #include <bonobo/bonobo-exception.h>
 #include <bonobo/bonobo-main.h>
 #include <bonobo/bonobo-object.h>
 #include <bonobo/bonobo-context.h>
-#include <bonobo/bonobo-shutdown.h>
+#include <bonobo/bonobo-private.h>
 
 #include <libintl.h>
 #include <string.h>
 
 #include <glib/gmain.h>
 
+GMutex                   *_bonobo_lock;
 CORBA_ORB                 __bonobo_orb = CORBA_OBJECT_NIL;
 PortableServer_POA        __bonobo_poa = CORBA_OBJECT_NIL;
 PortableServer_POAManager __bonobo_poa_manager = CORBA_OBJECT_NIL;
@@ -211,7 +212,8 @@ bonobo_init_full (int *argc, char **argv,
 
 	CORBA_exception_free (&ev);
 
-	bonobo_object_init ();
+	_bonobo_lock = g_mutex_new ();
+
 	bonobo_context_init ();
 
 	bindtextdomain (GETTEXT_PACKAGE, BONOBO_LOCALEDIR);
@@ -311,6 +313,81 @@ bonobo_main_quit (void)
 	g_main_loop_quit (bonobo_main_loops->data);
 }
 
+/**
+ * bonobo_poa_get_threadedv:
+ * @hint: the desired thread hint
+ * @args: va_args related to that hint
+ * 
+ * Get a predefined POA for a given threading policy/hint.  The
+ * returned POA can be passed as the "poa" constructor property of a
+ * #BonoboOject.
+ * 
+ * Return value: the requested POA.
+ **/
+PortableServer_POA
+bonobo_poa_get_threadedv (ORBitThreadHint hint,
+			  va_list         args)
+{
+	PortableServer_POA  poa;
+	CORBA_Environment   ev[1];
+	CORBA_PolicyList    policies;
+	CORBA_Object        policy_vals[1];
+	const char         *poa_name = NULL;
+
+#define MAP(a,b) \
+	case ORBIT_THREAD_HINT_##a: \
+		poa_name = b; \
+		break
+
+	switch (hint) {
+		MAP (NONE,           "BonoboPOAHintNone");
+		MAP (PER_OBJECT,     "BonoboPOAHintPerObject");
+		MAP (PER_REQUEST,    "BonoboPOAHintPerRequest");
+		MAP (PER_POA,        "BonoboPOAHintPerPOA");
+		MAP (PER_CONNECTION, "BonoboPOAHintPerConnection");
+		MAP (ONEWAY_AT_IDLE, "BonoboPOAHintOnewayAtIdle");
+		MAP (ALL_AT_IDLE,    "BonoboPOAHintAllAtIdle");
+#undef MAP
+	default:
+		g_assert_not_reached();
+	}
+
+	CORBA_exception_init (ev);
+
+	/* (Copy-paste from ORBit2/test/poa/poatest-basic08.c) */
+
+	policies._length = 1;
+	policies._buffer = policy_vals;
+	policies._buffer[0] = (CORBA_Object)
+		PortableServer_POA_create_thread_policy
+			(bonobo_poa (),
+			 PortableServer_ORB_CTRL_MODEL,
+			 ev);
+
+	poa = bonobo_poa_new_from (__bonobo_poa,
+				   poa_name, &policies, ev);
+
+	CORBA_Object_release (policies._buffer[0], NULL);
+
+	if (ev->_major == CORBA_NO_EXCEPTION)
+		ORBit_ObjectAdaptor_set_thread_hintv
+			((ORBit_ObjectAdaptor) poa, hint, args);
+
+	else {
+		if (strcmp (CORBA_exception_id (ev),
+			    ex_PortableServer_POA_AdapterAlreadyExists) == 0) {
+			CORBA_exception_free (ev);
+			poa = PortableServer_POA_find_POA (bonobo_poa (),
+							   poa_name,
+							   CORBA_FALSE, ev);
+		}
+	}
+	CORBA_exception_free (ev);
+	if (!poa)
+		g_warning ("Could not create/get poa '%s'", poa_name);
+
+	return poa;
+}
 
 /**
  * bonobo_poa_get_threaded:
@@ -322,75 +399,38 @@ bonobo_main_quit (void)
  * 
  * Return value: the requested POA.
  **/
-PortableServer_POA bonobo_poa_get_threaded (BonoboThreadHint hint)
+PortableServer_POA
+bonobo_poa_get_threaded (ORBitThreadHint hint, ...)
 {
-       PortableServer_POA  poa;
-       CORBA_Environment   ev;
-       CORBA_PolicyList   *policies;
-       const char         *poa_name = NULL;
-       ORBitThreadHint     thread_hint = ORBIT_THREAD_HINT_NONE;
+	va_list args;
+	PortableServer_POA poa;
 
-       switch (hint)
-       {
-       case BONOBO_POA_ALL_AT_IDLE:
-	       poa_name = "BonoboPOAAllAtIdle";
-	       thread_hint = ORBIT_THREAD_HINT_ALL_AT_IDLE;
-	       break;
-       case BONOBO_POA_ONEWAY_AT_IDLE:
-	       poa_name = "BonoboPOAOnewayAtIdle";
-	       thread_hint = ORBIT_THREAD_HINT_ONEWAY_AT_IDLE;
-	       break;
-       case BONOBO_POA_THREAD_PER_REQUEST:
-	       poa_name = "BonoboPOAThreadPerRequest";
-	       thread_hint = ORBIT_THREAD_HINT_PER_REQUEST;
-	       break;
-       default:
-	       g_assert_not_reached();
-       }
+	va_start (args, hint);
+	poa = bonobo_poa_get_threadedv (hint, args);
+	va_end (args);
 
-       CORBA_exception_init (&ev);
+	return poa;
+}
 
-       /* (Copy-paste from ORBit2/test/poa/poatest-basic08.c) */
+PortableServer_POA
+bonobo_poa_new_from (PortableServer_POA      template,
+		     const char             *name,
+		     const CORBA_PolicyList *opt_policies,
+		     CORBA_Environment      *opt_ev)
+{
+	PortableServer_POA poa;
+	CORBA_Environment real_ev[1], *ev;
 
-       /*
-        * Create child POA with MULTIPLE_ID Object Id Uniqueness policy and
-        * IMLICIT_ACTIVATION Implicit Activation policy.
-        */
-       policies           = CORBA_PolicyList__alloc ();
-       policies->_maximum = 2;
-       policies->_length  = 2;
-       policies->_buffer  = CORBA_PolicyList_allocbuf (2);
-       CORBA_sequence_set_release (policies, CORBA_TRUE);
-       policies->_buffer[0] = (CORBA_Policy)
-               PortableServer_POA_create_id_uniqueness_policy (
-                       bonobo_poa (),
-                       PortableServer_MULTIPLE_ID,
-                       &ev);
-       policies->_buffer[1] = (CORBA_Policy)
-               PortableServer_POA_create_implicit_activation_policy (
-                       bonobo_poa (),
-                       PortableServer_IMPLICIT_ACTIVATION,
-                       &ev);
+	if (!opt_ev)
+		CORBA_exception_init ((ev = real_ev));
+	else
+		ev = opt_ev;
 
-       poa = PortableServer_POA_create_POA (bonobo_poa (), poa_name,
-                                            bonobo_poa_manager (), policies, &ev);
-       CORBA_free (policies);
+	poa = ORBit_POA_new_from (bonobo_orb(),
+				  template, name, opt_policies, ev);
 
-       if (ev._major == CORBA_NO_EXCEPTION) {
-               ORBit_ObjectAdaptor_set_thread_hint ((ORBit_ObjectAdaptor) poa,
-						    thread_hint);
-       } else {
-               if (strcmp (CORBA_exception_id (&ev),
-                           ex_PortableServer_POA_AdapterAlreadyExists) == 0)
-               {
-                       CORBA_exception_free (&ev);
-                       poa = PortableServer_POA_find_POA (bonobo_poa (),
-                                                          poa_name,
-                                                          CORBA_FALSE, &ev);
-               }
-       }
-       CORBA_exception_free (&ev);
-       if (!poa)
-               g_warning ("Could not create/get poa '%s'", poa_name);
-       return poa;
+	if (!opt_ev)
+		CORBA_exception_free (real_ev);
+
+	return poa;
 }
